package ShipIt;
use strict;
use vars qw($VERSION);
use ShipIt::Conf;
use ShipIt::State;
use ShipIt::VC;
use ShipIt::Util;

$VERSION = '1.005';

=head1 NAME

ShipIt - software release tool

=head1 SYNOPSIS

 shipit

=head1 OVERVIEW
=cut
my $somecode = 1;

1;
